/*
  Name: Global.cpp
  Copyright: Polecatsoft 2005
  Game: Mercenary Soldier
  Author: Master Lenman
  Date: -
  Description: This file contains the global class,
	  that contains all globally accessible variables
*/

#include <allegro.h>
#include <libnet.h>
#include <fstream>
#include <string>
using namespace std;

#include "Global.h"
#include "Errors.h"
#include "ElementaryFunctions.h"

globals global;

int globals::load()
{
	ifstream readfile("settings", ios::binary);
	if(readfile.fail())
		return 0;

	readfile.read(&curmode, 1);
	if(!curmode)
#ifndef _DEBUG
		curmode = GFX_AUTODETECT_FULLSCREEN;
#else
		curmode = GFX_AUTODETECT_WINDOWED;
#endif

	readfile.read(&vsync, 1);

	readfile.read(i2ch(keyLeft), sizeof(keyLeft));
	readfile.read(i2ch(keyRight), sizeof(keyRight));
	readfile.read(i2ch(keyUp), sizeof(keyUp));
	readfile.read(i2ch(keyDown), sizeof(keyDown));
	readfile.read(i2ch(keyUse), sizeof(keyUse));
	readfile.read(i2ch(keyBoost), sizeof(keyBoost));
	readfile.read(i2ch(keyShoot), sizeof(keyShoot));
	readfile.read(i2ch(keyNextweapon), sizeof(keyNextweapon));
	readfile.read(i2ch(keyPreviousweapon), sizeof(keyPreviousweapon));
	readfile.read(i2ch(keyAim), sizeof(keyAim));
	readfile.read(i2ch(keyMission), sizeof(keyMission));
	readfile.read(i2ch(keyMessages), sizeof(keyMessages));

	readfile.read(&frameskip, 1);

	readfile.read(b2ch(useVideoRAM), sizeof(bool));

	readfile.read(i2ch(depth), sizeof(depth));

	readfile.read((char *)&sfxVol, 1);
	readfile.read((char *)&musicVol, 1);

	readfile.read((char *)&complexSound, sizeof(bool));
	readfile.read(i2ch(shotnumber), sizeof(shotnumber));

	readfile.close();

	return 1;
}

void globals::save()
{
	ofstream writefile("settings", ios::binary);

	if(!curmode)
#ifndef _DEBUG
		curmode = GFX_AUTODETECT_FULLSCREEN;
#else
		curmode = GFX_AUTODETECT_WINDOWED;
#endif
	writefile.write(&curmode, 1);
	writefile.write(&vsync, 1);

	writefile.write(i2ch(keyLeft), sizeof(keyLeft));
	writefile.write(i2ch(keyRight), sizeof(keyRight));
	writefile.write(i2ch(keyUp), sizeof(keyUp));
	writefile.write(i2ch(keyDown), sizeof(keyDown));
	writefile.write(i2ch(keyUse), sizeof(keyUse));
	writefile.write(i2ch(keyBoost), sizeof(keyBoost));
	writefile.write(i2ch(keyShoot), sizeof(keyShoot));
	writefile.write(i2ch(keyNextweapon), sizeof(keyNextweapon));
	writefile.write(i2ch(keyPreviousweapon), sizeof(keyPreviousweapon));
	writefile.write(i2ch(keyAim), sizeof(keyAim));
	writefile.write(i2ch(keyMission), sizeof(keyMission));
	writefile.write(i2ch(keyMessages), sizeof(keyMessages));

	writefile.write(&frameskip, 1);
	writefile.write(b2ch(useVideoRAM), sizeof(bool));
	writefile.write(i2ch(depth), sizeof(depth));

	writefile.write((char *)&sfxVol, 1);
	writefile.write((char *)&musicVol, 1);
	writefile.write((char *)&complexSound, sizeof(bool));
	writefile.write(i2ch(shotnumber), sizeof(shotnumber));
	writefile.close();
}

void globals::create()
{
#ifndef _DEBUG
	curmode = GFX_AUTODETECT_FULLSCREEN;
#else
	curmode = GFX_AUTODETECT_WINDOWED;
#endif
	vsync = true;
	keyLeft = KEY_A;
	keyRight = KEY_D;
	keyUp = KEY_W;
	keyDown = KEY_S;
	keyUse = KEY_SPACE;
	keyBoost = KEY_ALT;
	keyShoot = KEY_LCONTROL;
	keyNextweapon = KEY_E;
	keyPreviousweapon = KEY_Q;
	keyAim = KEY_LSHIFT;
	keyMission = KEY_TAB;
	keyMessages = KEY_M;
	frameskip = 4;
	sfxVol = 255;
	musicVol = 175;
	useVideoRAM = false;
	depth = desktop_color_depth();
	if(!depth)
		depth = 16;
	firstTime = true;
	shotnumber = 0;

	complexSound = true;

	curDriver = 0;

	save();
}

void globals::unload()
{
	destroy_bitmap(doublebuffer);
	destroy_bitmap(mainmenu);
	destroy_bitmap(crosshair);
	unload_datafile(datafile);
	save();
}

void globals::init()
{
	//Initialise variables that do not need to be
	//loaded from an external file

	debugMode = false;
	firstTime = false;
	showBlood = false;

	//Load variables from external files
	if(!load())
		create();

	//Set color depth
	set_color_depth(depth);

	//Set a graphics mode
	setMode(curmode);

	doublebuffer = createBitmap(SCREEN_W, SCREEN_H);
	if(!global.doublebuffer)
		errors::createError();

	mainmenu = loadBitmap("gfx/mainmenu.tga");
	
	crosshair = loadBitmap("gfx/crosshair.tga");

	datafile = load_datafile("data/data.dat");
	if(!datafile)
		errors::datafileError("data/data.dat");

	size20 = (FONT*)datafile[0].dat;
	size12 = (FONT*)datafile[1].dat;
	size12bold = (FONT*)datafile[2].dat;

	textout_right_ex(mainmenu, global.size12, allegro_id, SCREEN_W - 3, SCREEN_H - 6 - 2 * text_height(global.size12), makecol(0xFF, 0xFF, 0xFF), -1);
	if(VERSION_LETTER == 'A' || VERSION_LETTER == 'B')
		textout_right_ex(mainmenu, global.size12, (VERSIONSTRING
#ifdef _DEBUG
		+ ", debug version"
#endif
		).c_str(), SCREEN_W - 3, SCREEN_H - 3 - text_height(global.size12), makecol(0xFF, 0xFF, 0xFF), -1);

	//Network initialisation
	if(net_init())
		errors::networkError("Failed to initialise network code!");
	//load network configuration if availible
	net_loadconfig(NULL);
	//Detect drivers
	NET_DRIVERLIST drivers = net_detectdrivers(net_classes[NET_CLASS_INET].drivers);
	NET_DRIVERNAME *drivernames = net_getdrivernames(drivers);
	
	//Use first detected driver
	curDriver = drivernames[0].num;
	free(drivernames);

	if(curDriver < 0 || !net_initdriver(curDriver))
	{
		errors::networkError("No network driver found!");
	}
}

//Toggle between Fullscreen and windowed mode
void globals::toggleMode()
{
	//If mode is auto, make curmode WINDOWED so it will be changed to fullscreen
	if(!curmode)
#ifndef _DEBUG
		curmode = GFX_AUTODETECT_WINDOWED;
#else
		curmode = GFX_AUTODETECT_FULLSCREEN;
#endif
	curmode ^= 3; //XOR with 3 to switch between graphic modes

	setMode(curmode);
}

globals &globals::operator=(globals &copy)
{
	curmode = copy.curmode;
	debugMode = copy.debugMode;
	depth = copy.depth;
	keyBoost = copy.keyBoost;
	keyDown = copy.keyDown;
	keyAim = copy.keyAim;
	keyLeft = copy.keyLeft;
	keyMessages = copy.keyMessages;
	keyMission = copy.keyMission;
	keyNextweapon = copy.keyNextweapon;
	keyPreviousweapon = copy.keyPreviousweapon;
	keyRight = copy.keyRight;
	keyShoot = copy.keyShoot;
	keyUp = copy.keyUp;
	keyUse = copy.keyUse;
	musicVol = copy.musicVol;
	sfxVol = copy.sfxVol;
	useVideoRAM = copy.useVideoRAM;
	vsync = copy.vsync;
	complexSound = copy.complexSound;
	shotnumber = copy.shotnumber;
	showBlood = copy.showBlood;

	return copy;
}
