/*
  Name: Global.h
  Copyright: Polecatsoft 2005
  Game: Mercenary Soldier
  Author: Master Lenman
  Date: -
  Description: This file contains the global class,
	  that contains all globally accessible variables
*/

#ifndef HEADER_GLOBALS
#define HEADER_GLOBALS

#include <allegro.h>

class globals
{
private:
	//Loads values from a file
	int load();

public:
	//Constructor
	globals() { mainmenu = NULL; }

	//Destructor
	void unload();

	//Initialises some variables and calls the load function
	void init();

	//Creates a config File with standard values
	void create();

	//Saves the values to a file
	void save();

	//Toggles between fullscreen and windowed mode
	void toggleMode();

	//Copy constructor, used in options menu
	globals &operator=(globals &copy);


	//This bitmap is the background for all menues
	BITMAP *mainmenu;

	//Current graphicsmode
	char curmode;
	//Shall vsync be used?
	char vsync;

	//DoubleBuffer, to make the game run smoother
	BITMAP *doublebuffer;

	//Fonts
	FONT *size20, *size12, *size12bold;

	//Datafile, contains both fonts and the cursor
	DATAFILE *datafile;

	//Controls
	int keyLeft, keyRight, keyUp, keyDown, keyUse, keyBoost, keyShoot,
		keyNextweapon, keyPreviousweapon, keyAim, keyMission, keyMessages;

	//Frameskipping for slow computers
	char frameskip;

	//Is debug mode on?
	bool debugMode;

	//If you got enough RAM in your video card it might be faster to load
	//all bitmaps to the video card
	bool useVideoRAM;

	//32 or 16 bit mode?
	int depth;

	//Is set to true if the settings file was created
	bool firstTime;

	//Volume of sound effects and music
	unsigned char sfxVol, musicVol;

	//Shall 3D Sound be used?
	bool complexSound;

	//Screenshot count
	int shotnumber;

	//Shall blood be shown?
	bool showBlood;
	
	//Current network driver
	int curDriver;
	
	//The crosshair bitmap
	BITMAP *crosshair;
};

extern globals global;

#endif
