/*
  Name: MapEditor.h
  Copyright: Polecatsoft 2005
  Game: Mercenary Soldier
  Author: Master Lenman
  Date: -
  Description: The map editor is used to, well, to edit maps
*/

#ifndef EDITOR_CLASS
#define EDITOR_CLASS

#include <allegro.h>
#include <string>
#include <fstream>
#include "Unit.h"

class edUnit : public Unit
{
private:
	int type;
public:
	void setType(int type);
	inline int getType();
};

class editorMapClass
{
private:
	char *graphicData;
	int width, height;
	int size;
	BITMAP *tileset;
	std::string tilesetPath;
	int tileTypes;
	int visibleTilesX,
		visibleTilesY;
	int maxX,
		maxY;

	int refreshCountdown;

	char mapType;

	BITMAP *preview;
	int smallSize;

	std::string *scriptPaths;
	std::string mapPath;

	void setTileset(std::string filename);
	inline void drawTile(int x, int y, char tile, int tilesize, BITMAP *dest);
	void drawTile(int x, int y, char tile);
public:
	inline int getOffset(int x, int y);
	void loadMap(std::ifstream &readfile, std::string filename);
	void draw(int &posX, int &posY, int replacePos, int &replaceGraphic);
	void draw(int &posX, int &posY);
	int getMapWidth();
	int getMapHeight();

	void save(std::ofstream &writefile, bool compiled);

	void setTile(int posX, int posY, int newGraphic);

	std::string getMapPath();

	void poll();

	std::string &getScriptPath(int offset);
	std::string &getScriptPath(int tileX, int tileY);

	BITMAP *getMapPreview(int smallSizeX = 400, int smallSizeY = 300);

	int getSmallSize();
};

class mapEditor
{
private:
	editorMapClass map;

	//Contains all civilians and enemies
	Unit basetype;
	int unitsUsed;
	edUnit units[1024];
	std::string scriptPaths[1024];
	std::string charsetPath;
	int playerTypes, enemyTypes, civilianTypes;

	bool campaign;
	std::string campaignName;

	bool save(bool compiled = false);
	bool saveCompiled();
public:
	mapEditor(std::string mapPath);
	void operator()();
};

#endif
