/*
  Name: SMPgame.cpp
  Copyright: Polecatsoft 2005
  Game: Mercenary Soldier
  Author: Master Lenman
  Date: -
  Description: This is the server for multiplayer games.
    It contains one instance of a multiplayer game for the player
	playing on the server.
*/

#include <allegro.h>
#include <almp3.h>
#include <libnet.h>
#include <string>
#include <fstream>
#include <cmath>
using namespace std;

#include "ElementaryFunctions.h"
#include "requests.h"
#include "Global.h"
#include "keyStatesClass.h"
#include "mapClass.h"
#include "bullet.h"
#include "SMPgame.h"
#include "Errors.h"
#include "CMPgame.h"
#include "mpUnit.h"
#include "packet.h"

//Maximum number of bullets that can be created
const int MAX_BULLETS = 1024;

//string weapons[6] = { "Pistol", "Shotgun", "MG", "Sniper", "Grenades", "Rocket Launcher" };

const int
TYPE_PLAYER = 0,
TYPE_BULLET = -1;

int serverMultiplayerGame::operator()()
{
	int i;
	int retNum;
	int curSkip = 0;
	bool running = true;
	int slowMessageDisplayed = 5;
	int quitWaitTime = 0;
	int lastContact[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
	messageNeededCountdown = 16;
	//timerCounter = 2;
	int graphCountdown = 4;
	while(running)
	{
////////////////////////////////////////////////////////////
/////////////////////// LOGIC //////////////////////////////
////////////////////////////////////////////////////////////
		while(timerCounter > 0 && running)
		{
            checkConnections();

			if(!dedicatedServer)
			{
				game->exeLogic();
			}
			//Adjust 8 to maximum number of players ever possible
			char sendUpdateFlag = 0;
			//Get move Requests from all Units
			for(i = 0; i < maxPlayers; ++i)
			{
				if(players[i].isActive())
				{
					int length;
					bool once = false;
					//Get all packets in qeue
					while(length = pack.receive(players[i].getConnection()))
					{
						once = true;
						lastContact[i] = 0;
						char identifier = pack.get();
						switch(identifier)
						{
						case P_MOVEREQ:
							{
								struct mpRequest moveRequest;
								pack.read(reinterpret_cast<char*>(&moveRequest), sizeof(mpRequest));
								players[i].getPhase() = moveRequest.phase;
								players[i].getState() = moveRequest.curSprite;
								players[i].getDirection() = moveRequest.dir;
								
								if(moveRequest.moveRequestFlag & 1)
								{
									int backWards = 128 * ((moveRequest.moveRequestFlag & 2) >> 1);
									float newX, newY;
									newX = std::sin(deg2rad(players[i].getDirection())) * 3.0;
									newY = std::cos(deg2rad(players[i].getDirection())) * -3.0;
									if(moveRequest.moveRequestFlag & 2)
									{
										newX *= -0.75;
										newY *= -0.75;
									}
									newX += players[i].getFlPosX();
									newY += players[i].getFlPosY();
									if(map.handleMoveRequest(newX, newY, players[i].getFlPosX(), players[i].getFlPosY()))
									    handleMoveRequest(newX, newY, players[i].getFlPosX(), players[i].getFlPosY(), i);
									players[i].getFlPosX() = newX;
									players[i].getFlPosY() = newY;
								}
								sendUpdateFlag |= (1 << i);
							}
							break;
							//New player requesting to join
						case P_REQUEST:
							if(pack.get() == MAJOR_VERSION && pack.get() == MINOR_VERSION && pack.get() == VERSION_LETTER)
							{
								char nameBuffer[9];
								pack.read(nameBuffer, 8);
								nameBuffer[8] = 0;
								players[i].name = nameBuffer;
								assignPosition(i);
								int j;
								for(j = 0; j < maxPlayers; ++j)
								{
									if(j == i)
									    continue;
									if(players[i].name.empty() || players[i].name == players[j].name)
									{
									    players[i].name = "Player " + i2str(i + 1);
									    break;
									}
								}
								pack.reset();
								pack.put(P_ACCEPT);
								pack.put(i);
								pack.put(maxPlayers);
								//max life and armor
								pack.put(100);
								pack.put(100);
								pack.write(mapname.c_str(), mapname.length());
								pack.put(0);
								pack.send(players[i].getConnection());
								messages->sendText(string("New player joined: ") + players[i].name
									+ string(" (") + net_getpeer(players[i].getConnection()) + string(")"));
								messages->output();
								//Tell all players that a new player has joined
								//and tell the new player all existing players
								pack.reset();
								pack.put(P_NEWPLAYER);
								pack.put(i);
								pack.put(0);
								pack.write(players[i].name.c_str(), players[i].name.length());
								int k;
								for(k = players[i].name.length(); k < 8; ++k)
								    pack.put(0);
								//New player
								pack.put(1);
								serverBroadcast();
								//Tell the new player about all existing players
								for(j = 0; j < maxPlayers; ++j)
								{
									if(j == i || !players[j].isActive())
									    continue;
									pack.reset();
									pack.put(P_NEWPLAYER);
									pack.put(j);
									pack.put(players[j].getPoints());
									pack.write(players[j].name.c_str(), players[j].name.length());
									for(k = players[j].name.length(); k < 8; ++k)
									    pack.put(0);
									//No new player
									pack.put(0);
									pack.send(players[i].getConnection());
        						}

        						//Send new positions to all players
        						for(j = 0; j < maxPlayers; ++j)
        						    if(players[j].isActive()) sendUpdateFlag |= (1 << j);
							}
							else
							{
								pack.reset();
								pack.put(P_VERSIONMISMATCH);
								pack.send(players[i].getConnection());
								players[i].leaveServer();
							}
							break;
						case P_SERVERSHUTDOWN:
							messages->sendText(players[i].name + " left the server.");
							messages->output();
							players[i].leaveServer();
							--playersJoined;
							pack.reset();
							pack.put(P_PLAYERLEFT);
							pack.put(i);
							serverBroadcast();
							break;
						};
					}
					if(!once)
					{
						++lastContact[i];
						if(lastContact[i] > 64)
						{
							players[i].leaveServer();
							--playersJoined;
							pack.reset();
							pack.put(P_PLAYERLEFT);
							pack.put(i);
							serverBroadcast();
							messages->sendText("Lost connection to player ");
							messages->sendInt(i + 1);
							messages->sendText(string(" (") + players[i].name + string(")"));
							messages->output();
						}
					}
					if(!(--messageNeededCountdown))
					{
						pack.reset();
						pack.put(P_NOP);
						serverBroadcast();
						messageNeededCountdown = 32;
					}
				}
				else
					lastContact[i] = 0;
			}
			//Send update of player positions to all players
			pack.reset();
			pack.put(P_POSUPDATE);
			pack.put(sendUpdateFlag);
			for(i = 0; i < maxPlayers; ++i)
			{
				if(!(sendUpdateFlag & (1 << i)))
				    continue;
				pack.write(f2ch(players[i].getFlPosX()), sizeof(float));
				pack.write(f2ch(players[i].getFlPosY()), sizeof(float));
				pack.put(players[i].getState());
				pack.put(players[i].getDirection());
				pack.put(players[i].getPhase());
			}
			if(sendUpdateFlag)
			    serverBroadcast();

			//Update map animation
			map.poll();
			if(dedicatedServer)
				messages->poll();

			if(keyStates[KEY_ESC])
			{
				if(quitWaitTime--)
				    running = false;
				else
				    quitWaitTime = 64;
			}
			else if(quitWaitTime)
				--quitWaitTime;
				
			if(graphCountdown)
			    --graphCountdown;

			--timerCounter;
			++curSkip;
			if(curSkip >= global.frameskip)
			{
				timerCounter = 0;
				if(!(--slowMessageDisplayed))
				{
					slowMessageDisplayed = 20;
					messages->sendText("Your computer is not fast enough to be server.\n"
						"Please consider using a faster computer as server.");
					messages->output();
				}
			}
			if(!running)
			{
				pack.reset();
				pack.put(P_SERVERSHUTDOWN);
				serverBroadcast();
				for(i = 0; i < maxPlayers; ++i)
				    if(players[i].isActive()) players[i].leaveServer();
			}
		}
////////////////////////////////////////////////////////////
/////////////////////// GRAPHIC ////////////////////////////
////////////////////////////////////////////////////////////
		curSkip = 0;
		if(graphCountdown)
			continue;
		clear(global.doublebuffer);
		if(dedicatedServer)
		{
			messages->draw(0, true);
		}
		else
		    game->exeGraphic();

		if(key[global.keyMission])
		{
			int active = 0;
			for(i = 0; i < maxPlayers; ++i)
			{
				if(!players[i].isActive())
				    continue;
				++active;
			}
			rectfill(global.doublebuffer, 60, 60, 740, (active + 1) * 20 + 80, 0);
			textout_centre_ex(global.doublebuffer, global.size20, (
			string("Mercenary Soldier Server, ") + i2str(active) + " of " + i2str(maxPlayers) + " players"
			).c_str(), SCREEN_W >> 1, 60, makecol(0xFF, 0xFF, 0xFF), -1);
			active = 0;
			for(i = 0; i < maxPlayers; ++i)
			{
				if(!players[i].isActive())
				    continue;
				FONT *useFont = global.size12;
				if(game->getmyID() == i)
				    useFont = global.size12bold;
				textout_centre_ex(global.doublebuffer, useFont, (
				players[i].name + string(" (") + net_getpeer(players[i].getConnection()) + ") - "
				+ i2str(players[i].getPoints()) + " points").c_str(), SCREEN_W >> 1, active * 19 + 90, makecol(0xFF, 0xFF, 0xFF), 0);
				++active;
			}
		}
		if(quitWaitTime)
		    textout_centre_ex(global.doublebuffer, global.size20, "Press esc again to quit.", SCREEN_W >> 1, 20, makecol(0xFF, 0xFF, 0xFF), 0);
		if(global.vsync)
			vsync();
		show();
	}
	return retNum;
}

void serverMultiplayerGame::handleMoveRequest(float &newPosX, float &newPosY, float oldPosX, float oldPosY, int curPlayer)
{
 	for(int i = 0; i < maxPlayers; ++i)
	{
		if(!players[i].isActive() || !players[i].isAlife() || curPlayer == i)
			continue;
		//5 is the radius
		if(collides(int(newPosX) - RADIUS, int(newPosY) - RADIUS, int(newPosX) + RADIUS, int(newPosY) + RADIUS,
			players[i].getPosX() - RADIUS, players[i].getPosY() - RADIUS, players[i].getPosX() + RADIUS, players[i].getPosY() + RADIUS))
		{
			newPosX = oldPosX;
			newPosY = oldPosY;
			return;
		}
	}
}

serverMultiplayerGame::~serverMultiplayerGame()
{
	delete [] spawnPointsX;
	delete [] spawnPointsY;
	delete [] spawnPointsDir;
	delete [] spawnPointsTeam;
	delete [] players;
	if(!dedicatedServer)
		delete game;
	net_closeconn(listenConn);
	delete [] bullets;
	delete messages;
}

serverMultiplayerGame::serverMultiplayerGame(string name, const int maxPlayersVal, const int playersToStart, const bool dedicated)
: maxPlayers(maxPlayersVal), minPlayers(playersToStart), dedicatedServer(dedicated)
{
	if(maxPlayersVal > 8)
	    errors::multiplayerError("A maximum of 8 players is allowed!");
	if(playersToStart > maxPlayersVal)
	    errors::multiplayerError("Minimum players is higher than maximum players!");
	//Show a message telling the user that the map is beeing loaded
	clear(global.doublebuffer);
	textout_centre_ex(global.doublebuffer, global.size20, "Initialising server, please wait...", SCREEN_W >> 1, SCREEN_H >> 1, makecol(0xFF, 0xFF, 0xFF), -1);
	show();

	bullets = new bullet[MAX_BULLETS];

	mapname = name;
	string mapPath = string("mpmaps/") + name + string(".msm");
	ifstream readfile(mapPath.c_str(), ios::binary);
	//ifstream readfile("maps/map1.msm", ios::binary);
	if(readfile.fail())
		errors::mapLoadError(mapPath, "Failed to open file");
	gameType = map.loadMap(readfile, mapPath);

	char cBuffer;
	//Global script
	readfile.read(&cBuffer, 1);
	if(cBuffer)
	    errors::mapLoadError(mapPath, "Global scripts in multiplayer maps are not allowed!");

	//Campaign information
	readfile.read(&cBuffer, 1);
	if(cBuffer)
	    errors::mapLoadError(mapPath, "Multiplayer maps must not belong to a campaign");

	//Characterset
	string charSetPath;
	getline(readfile, charSetPath, DELIMITER);

	//Create instances
	players = new mpUnit[maxPlayers];
	playersJoined = 0;

	//Load spawning points
	readfile.read(i2ch(nSpawnPoints), sizeof(int));
	if(nSpawnPoints < 1)
		errors::mapLoadError(mapPath, "No spawn points on map!");
	spawnPointsX = new float[nSpawnPoints];
	spawnPointsY = new float[nSpawnPoints];
	spawnPointsDir = new int[nSpawnPoints];
	spawnPointsTeam = new char[nSpawnPoints];

	int i;
	for(i = 0; i < nSpawnPoints; ++i)
	{
		char buffer;
		readfile.read(&(spawnPointsTeam[i]), 1);
		readfile.read(f2ch(spawnPointsX[i]), sizeof(float));
		readfile.read(f2ch(spawnPointsY[i]), sizeof(float));
		readfile.read(&buffer, 1);
		spawnPointsDir[i] = buffer;
		readfile.read(&buffer, 1);
		readfile.read(&buffer, 1);
		if(buffer)
		    errors::mapLoadError(mapPath, "Scripts in multiplayer maps are not allowed!");
	}
	readfile.close();

	int maxV = 32;
	if(!dedicated)
	    maxV = 7;
	messages = new messageClass(32, maxV, 3, global.size12, global.size12bold, global.doublebuffer);

	//Handle network stuff
	listenConn = net_openconn(global.curDriver, "");
	if(!listenConn)
		errors::networkError("Cannot open listening connection!");
	if(net_listen(listenConn))
	    errors::networkError("Cannot listen on listening connection!");

	if(!dedicatedServer)
	{
		game = new clientMultiplayerGame("127.0.0.1");
		game->setMessageClass(messages);
	}
	else
	{
		//Let game run in background
		if(set_display_switch_mode(SWITCH_BACKGROUND))
			errors::fatalError("Failed to switch into background mode!");
	}
	
	curSpawnPoint = 0;
}

void serverMultiplayerGame::addBullets(bulletRequest *bReq)
{
	if(bReq)
	{
		int j = 0, i;
		for(i = 0; i < BULLETS[bReq->getType()]; ++i)
		{
			for( ; j < MAX_BULLETS; ++j)
			{
				if(!bullets[j].isActive())
				{
					int spread[2] = { 1, -1 };
					int rd = rand();
					int direction;
					if(BULLETS[bReq->getType()] > 1)
					{
						direction = bReq->getDir() - (SPREAD[bReq->getType()] + 2) + (SPREAD[bReq->getType()] << 1) / BULLETS[bReq->getType()] * (i + 1);
					}
					else if(SPREAD[bReq->getType()])
						direction = bReq->getDir() + (rd % SPREAD[bReq->getType()]) * spread[rd & 1];
					else
						direction = bReq->getDir();
					bullets[j].init(bReq->getType(), direction, bReq->getPosX(), bReq->getPosY(), bReq->getTeam(), true);
					break;
				}
			}
		}
	}
}

void serverMultiplayerGame::explosion(bulletRequest bReq)
{
	int angle = 0;
	bulletSounds[bReq.getType()].play(0, global.sfxVol);
	//play_sample(bulletSounds[bReq.getType()], global.sfxVol, 128, 1000, 0);
	int j = 0;
	while(angle < 256)
	{
		for( ; j < MAX_BULLETS; ++j)
		{
			if(!bullets[j].isActive())
			{
				bullets[j].init(bReq.getType(), angle, bReq.getPosX(), bReq.getPosY(), 0, false);
				angle += SPLINTS[bReq.getType() - 2];
				break;
			}
		}
	}
}

bool serverMultiplayerGame::collides(int left1, int top1, int right1, int bottom1, int left2, int top2, int right2, int bottom2)
{
	bool collides[4] = { false, false, false, false };

	//Left side colliding
	if(left1 <= right2 && left1 >= left2)
	{
		collides[0] = true;
	}
	//Right side colliding
	if(right1 >= left2 && right1 <= right2)
	{
		collides[1] = true;
	}
	//Top side colliding
	if(top1 <= bottom2 && top1 >= top2)
	{
		collides[2] = true;
	}
	//Bottom side colliding
	if(bottom1 >= top2 && bottom1 <= bottom2)
	{
		collides[3] = true;
	}

	if(!(collides[0] || collides[1] || collides[2] || collides[3]))
		return false;

	//Check total collision for all 4 corners
	if(collides[0] && collides[2])
		return true;
	if(collides[0] && collides[3])
		return true;
	if(collides[1] && collides[2])
		return true;
	if(collides[1] && collides[3])
		return true;

	if(bottom1 - top1 > bottom2 - top2)
	{
		if((collides[0] || collides[1]) && top1 < top2 && bottom1 > bottom2)
			return true;
	}
	if(right1 - left1 > right2 - left2)
	{
		if((collides[2] || collides[3]) && left1 < left2 && right1 > right2)
			return true;
	}

	return false;
}

void serverMultiplayerGame::checkConnections()
{
	NET_CONN *newconn = net_poll_listen(listenConn);

	if(newconn)
	{
		int i;
		bool found = false;
		for(i = 0; i < maxPlayers; ++i)
		{
			if(!players[i].isActive())
			{
				found = true;
				players[i].getConnected(newconn);
				++playersJoined;
				break;
			}
		}
		if(!found)
		{
			pack.reset();
			pack.put(P_FULL);
			pack.send(newconn);
			net_closeconn(newconn);
		}
	}
}

void serverMultiplayerGame::serverBroadcast()
{
	int i;
	for(i = 0; i < maxPlayers; ++i)
	{
		if(players[i].isActive())
			pack.send(players[i].getConnection());
	}
	messageNeededCountdown = 32;
}

void serverMultiplayerGame::assignPosition(int player)
{
	++curSpawnPoint %= nSpawnPoints;
	players[player].getFlPosX() = spawnPointsX[curSpawnPoint];
	players[player].getFlPosY() = spawnPointsY[curSpawnPoint];
	players[player].getDirection() = spawnPointsDir[curSpawnPoint];
}
