/*
  Name: Unit.h
  Copyright: Polecatsoft 2005
  Game: Mercenary Soldier
  Author: Master Lenman
  Date: -
  Description: The unit class is derived from the Entity Class
    and is the parent class for all Units (Player, Enemies, Civilians)
*/

#ifndef UNITCLASS
#define UNITCLASS

#include <allegro.h>
#include <string>
#include "Entity.h"
#include "requests.h"

class Unit : public Entity
{
protected:
	int maxLife, life;
	int maxArmor, armor;
	int armorType;
	float speed;
	//An array of BITMAP pointers
	BITMAP **sprites;
	//Only destroy bitmaps if this is true
	bool destroy;
	//Needed for animating the run animation
	int phase;
	//Needed for blood animation
	int bloodPhase;

	bool enemyOrCivilian;

	//Needed for AI
	int destDir;
	int triggerDist;
	int shootFreq;

	//Time before Unit can shoot again
	int shootCooldown;

	int curWeapon;
public:
	Unit();
	~Unit();
	//Set information loaded from map File
	void initialise(float initPosX, float initPosY, int initDir, int initState);
	//Set information retrieved from character set
	void load(float speed, int maxLife, int maxArmor, int armType, std::string spriteData, int trigDist, int weapon = 0, int shootPause = 0);
	//Get request for moving
	virtual moveRequest *getMoveRequest(int playerPosX, int playerPosY);
	//Get request for shooting
	virtual bulletRequest *getBulletRequest();
	//Take back the handled requests
	void takeRequests(moveRequest *mReq, bulletRequest *bReq);
	//Get bitmaps and stuff from enemy base type
	void basetypeGet(Unit &basetype);

	//Get a specific sprite adress from the Unit
	BITMAP *getSpriteAddress(int offset);

	//Get hit by a bullet
	void getHit(int bulletType);

	//Draw only if alife
	virtual void draw(int cameraX, int cameraY);

	int getLifePercentage();
	int getArmorPercentage();

	void setState(int newState);

	int getTriggerDist();

	//The unit is a civilian after this function was called
	//void setCivilian();

	//Standard data returners
	float getSpeed();
	int getMaxarmor();
	int getMaxlife();
	bool isAlife();
	bool isEnemy();
	int getArmortype();
	int getWeapon();
	int getShootFreq();
};

#endif
