/*
  Name: VM.h
  Copyright: Polecatsoft 2005
  Game: Mercenary Soldier
  Author: Master Lenman
  Date: -
  Description: This file contains the Virtual machine which will execute byte code
	  passed to it. It will also return instructions to the game and the map
*/

#ifndef MSSCRIPT_VM
#define MSSCRIPT_VM

#include <string>
#include "Errors.h"
#include "messageClass.h"
#include "Sound.h"
#include "bytecode.h"

const int MAXSOUNDS = 16;

class VirtualMachine
{
private:
	int var[ADDRESSRANGE];
	int mapWidth, mapHeight;
	char *graphicData;
	inline int getOffset(int x, int y);
	std::string nextMap;
	bool needsNextMap;
	messageClass *messages;
	int objectives;

	Sound sounds[MAXSOUNDS];

	//Internal code variable for internal use
	bytecode *internalCode;
	//Returns the value of the variable 
	int getPar(bool varOnly = false);
	int getVarPar();
public:
	VirtualMachine();
	void mapSetup(int w, int h, char *data);
	void compute(bytecode &code);
	bool needNextMap();
	std::string getMapString();
	void setMessageClass(messageClass *msc);
	void setObjectives(int obj);
	void pollSounds();
	void updateSoundVol();
	inline int &getVar(int offset, bool write = false)
	{
		if(offset > ADDRESSRANGE)
			errors::scriptError("Illegal 'read' operation on protected memory!");
		if(write && offset < PROTECTEDRANGE)
		{
			//allegro_message("%i", offset);
			errors::scriptError("Illegal 'write' operation on protected memory!");
		}
		return var[offset];
	}
};

#endif
