/*
  Name: mapClass.h
  Copyright: Polecatsoft 2005
  Game: Mercenary Soldier
  Author: Master Lenman
  Date: -
  Description: This is the map class used in MP mode,
    it's the base class for the SP mode map class
*/

#ifndef MAPCLASS_DEF
#define MAPCLASS_DEF

#include <allegro.h>
#include <string>
#include <fstream>

//#include "bytecode.h"
//#include "VM.h"
#include "Global.h"

extern const int TILESIZE;
extern const int RADIUS;

class mapClass
{
protected:
	char *graphicData;
	char *hightData;
	char *followUpData;
	int width, height;
	int size;
	BITMAP *tileset;
	int tileTypes;
	int mapType;
	int visibleTilesX,
		visibleTilesY;
	int maxX,
		maxY;

	//BITMAP *preview;
	//int smallSize;

	//Count down for animating tiles
	int animCountdown;
	//Script code for all events
	//bytecode **onEnter;
	//bytecode **onLeave;
	//bytecode **duringTouch;
	//bytecode **onExplode;
	//bytecode **onUse;

	void setTileset(std::string filename);

	inline void drawTile(int x, int y, char tile, int tilesize, BITMAP *dest = global.doublebuffer)
	{
		blit(tileset, dest, tile * tilesize, 0, x, y, tilesize, tilesize);
	}
	inline int getOffset(int x, int y)
	{
		return x + width * y;
	}
	//Get the x and y positions that correspond to an offset
	void xyFromOffset(int &x, int &y, int offset);
public:
	inline mapClass()
	{
		graphicData = NULL;
		hightData = NULL;
		tileset = NULL;
	}
	~mapClass();
	char loadMap(std::ifstream &readfile, std::string filename);
	//Draw whole map to screen
	void draw(int &posX, int &posY);
	//Function to handle moving requests, returns false if the request was declined
	bool handleMoveRequest(float &newPosX, float &newPosY, float oldPosX, float oldPosY);

	//BITMAP *getMapPreview(int sizeX, int sizeY);
	//int getSmallSize();

	//Standard data returners
	int getMapWidth();
	int getMapHeight();

	const int getTilesize();

//	nodeList getPath(int startX, int startY, int endX, int endY);

	char *getGraphicDataAddress();

	//Poll for animation
	void poll();
};

#endif
