/*
  Name: MapEditor.cpp
  Copyright: Polecatsoft 2005
  Game: Mercenary Soldier
  Author: Master Lenman
  Date: -
  Description: The map editor is used to, well, to edit maps
*/

#include <allegro.h>
#include <libnet.h>
#include <string>
#include <fstream>
using namespace std;

#include "ElementaryFunctions.h"
#include "Errors.h"
#include "Entity.h"
#include "requests.h"
#include "Unit.h"
#include "Global.h"
#include "MapEditor.h"
#include "keyStatesClass.h"
#include "Compiler.h"

const int
TILESIZE = 32;

mapEditor::mapEditor(string mapPath)
{
	clear(global.doublebuffer);
	textout_centre_ex(global.doublebuffer, global.size20, "Loading map data, please wait...", SCREEN_W >> 1, SCREEN_H >> 1, makecol(0xFF, 0xFF, 0xFF), -1);
	show();

	ifstream readfile(mapPath.c_str(), ios::binary);
	if(readfile.fail())
		errors::mapLoadError(mapPath, "Failed to open file");

	map.loadMap(readfile, mapPath);

	char cBuffer;
	readfile.read(&cBuffer, 1);
	if(cBuffer)
		errors::editorError("Precompiled scripts are illegal for editing");

	readfile.read(&cBuffer, 1);
	campaign = cBuffer & 1;
	if(campaign)
		getline(readfile, campaignName, DELIMITER);

	getline(readfile, charsetPath, DELIMITER);
	ifstream charset(charsetPath.c_str());
	if(charset.fail())
		errors::mapLoadError(mapPath, "Failed to open set of characters");

	int i;

	string strBuffer;
	float fBuffer;
	int iBuffer;
	charset >> playerTypes;
	for(i = 0; i < playerTypes; ++i)
	{
		charset >> strBuffer >> fBuffer >> iBuffer >> iBuffer;
		if(iBuffer)
			charset >> iBuffer;
		for(int j = 0; j < 5; ++j)
			charset >> iBuffer;
		charset >> strBuffer;
	}
	charset >> enemyTypes;
	for(i = 0; i < enemyTypes; ++i)
	{
		charset >> strBuffer >> fBuffer >> iBuffer >> iBuffer;
		if(iBuffer)
			charset >> iBuffer;
		charset >> iBuffer >> strBuffer;
	}
	charset >> civilianTypes;
	charset.close();

	basetype.load(0.0, 1, 0, 0, "data/editorgr.dat", 0);
	readfile.read(i2ch(unitsUsed), sizeof(unitsUsed));
	for(i = 0; i < unitsUsed; ++i)
	{
		char type;
		readfile.read(&type, 1);
		float posX, posY;
		readfile.read(f2ch(posX), sizeof(posX));
		readfile.read(f2ch(posY), sizeof(posY));
		char angle;
		readfile.read(&angle, sizeof(angle));
		char initState;
		readfile.read(&initState, 1);
		char scriptInfo;
		readfile.read(&scriptInfo, 1);
		if(scriptInfo == 1)
			errors::editorError("Precompiled scripts are illegal for editing");
		else if(scriptInfo == 2)
			getline(readfile, scriptPaths[i], DELIMITER);

		int graphic;
		//Set different sprite colors for different unit types
		if(type < playerTypes)
			graphic = 0;
		else if(type < enemyTypes + playerTypes)
			graphic = 1;
		else
			graphic = 2;
		units[i].initialise(posX, posY, angle, 0);
		units[i].getState() = graphic;
		units[i].basetypeGet(basetype);
		units[i].setType(type);
	}
	//Also initialise unused units
	//They are initialised as the first type after the player
	int graphic, type;
	type = playerTypes;
	if(enemyTypes + playerTypes > type)
		graphic = 1;
	else
		graphic = 2;
	for( ; i < 1024; ++i)
	{
		units[i].initialise(0.0, 0.0, 0, 0);
		units[i].getState() = graphic;
		units[i].basetypeGet(basetype);
		units[i].setType(type);
	}

	readfile.close();
}

void mapEditor::operator ()()
{
	int i;
	//Vars for mapedit mode
	int curTileX = 0, curTileY = 0;
	int curGraphic = 0;

	//Vars for unitedit mode
	int curUnit = 0;

	const int MODE_MAPEDIT = 0,
		MODE_UNITEDIT = 1;
	int mode = MODE_MAPEDIT;

	int dividor = 8;

	while(true)
	{
		if(timerCounter > 0)
		{
			//Functions keys have in mapedit mode
			if(mode == MODE_MAPEDIT)
			{
				if(key[global.keyAim])
				{
					if(key[global.keyLeft] && curTileX)
						--curTileX;
					if(key[global.keyRight] && curTileX < map.getMapWidth() - 1)
						++curTileX;
					if(key[global.keyUp] && curTileY)
						--curTileY;
					if(key[global.keyDown] && curTileY < map.getMapHeight() - 1)
						++curTileY;
					if(keyStates[global.keyNextweapon])
						alert(("X: " + i2str(curTileX) + "/" + i2str(map.getMapWidth())).c_str(),
							("Y: " + i2str(curTileY) + "/" + i2str(map.getMapHeight())).c_str(), 0, "OK", 0, 0, 0);
						//allegro_message("X: %i/%i\nY: %i/%i", curTileX, map.getMapWidth(), curTileY, map.getMapHeight());
				}
				else
				{
					if(keyStates[global.keyLeft] && curTileX)
						--curTileX;
					if(keyStates[global.keyRight] && curTileX < map.getMapWidth() - 1)
						++curTileX;
					if(keyStates[global.keyUp] && curTileY)
						--curTileY;
					if(keyStates[global.keyDown] && curTileY < map.getMapHeight() - 1)
						++curTileY;
					if(keyStates[global.keyPreviousweapon] && curGraphic)
						--curGraphic;
					if(keyStates[global.keyNextweapon])
						++curGraphic;
					if(key[global.keyUse])
						map.setTile(curTileX, curTileY, curGraphic);
				}
				if(key[global.keyShoot])
				{
					if(alert("Use script for this tile?", "If a script was previously assigned to this tile,", "it will be replaced.", "Yes", "No", 0, 0) == 1)
					{
						if(!map.getScriptPath(curTileX, curTileY).length())
							map.getScriptPath(curTileX, curTileY) = "scripts/";
						if(!fileSelect("Please select a script.", map.getScriptPath(map.getOffset(curTileX, curTileY)), "ums;txt;/-d", SCREEN_W - 100, SCREEN_H - 100))
							map.getScriptPath(curTileX, curTileY) = "";
					}
					else
						map.getScriptPath(curTileX, curTileY) = "";
				}
			}
			//Functions keys have in unitedit mode
			if(mode == MODE_UNITEDIT)
			{
				if((keyStates[global.keyMission] && unitsUsed < 1024) || !unitsUsed)
				{
					if(alert("Create new unit?", 0, 0, "Yes", "No", 0, 0) == 1)
					{
						scriptPaths[unitsUsed] = "";
						++unitsUsed;
						curUnit = unitsUsed - 1;
					}
					else if(!unitsUsed)
						mode = MODE_MAPEDIT;
				}
				//These are only availible when there are units on the map
				if(unitsUsed)
				{
					if(!key[global.keyAim])
					{
						if(keyStates[KEY_F5] && dividor > 1)
							--dividor;
						if(keyStates[KEY_F6])
							++dividor;
						if(keyStates[global.keyPreviousweapon] && curUnit)
							--curUnit;
						if(keyStates[global.keyNextweapon] && curUnit < unitsUsed - 1)
							++curUnit;
						if(key[global.keyLeft] && units[curUnit].getFlPosX() >= TILESIZE / 8)
							units[curUnit].getFlPosX() -= TILESIZE / dividor;
						if(key[global.keyRight] && units[curUnit].getFlPosX() <= map.getMapWidth() * TILESIZE - TILESIZE / 8)
							units[curUnit].getFlPosX() += TILESIZE / dividor;
						if(key[global.keyUp] && units[curUnit].getFlPosY() >= TILESIZE / 8)
							units[curUnit].getFlPosY() -= TILESIZE / dividor;
						if(key[global.keyDown] && units[curUnit].getFlPosY() <= map.getMapHeight() * TILESIZE - TILESIZE / 8)
							units[curUnit].getFlPosY() += TILESIZE / dividor;
						if(keyStates[global.keyUse])
						{
							if(alert("Delete current unit?", 0, 0, "Yes", "No", 0, 0) == 1)
							{
								--unitsUsed;
								scriptPaths[curUnit] = scriptPaths[unitsUsed];
								units[curUnit].initialise(units[unitsUsed].getFlPosX(), units[unitsUsed].getFlPosY(), units[unitsUsed].getDirection(), units[unitsUsed].getState());
								units[curUnit].setType(units[unitsUsed].getType());
								//int passed = 0;
								//for(i = 0; i < unitsUsed; ++i)
								//{
								//	if(i == curUnit)
								//		passed = 1;
								//	scriptPaths[i + passed] = scriptPaths[i + passed + 1];
								//	units[i + passed].initialise(units[i + passed + 1].getFlPosX(), units[i + passed + 1].getFlPosY(), units[i + passed + 1].getDirection(), units[i + passed + 1].getState());
								//	units[i + passed].setType(units[i + passed + 1].getType());
								//}
							}
						}
						if(keyStates[global.keyShoot])
						{
							if(alert("Use script for this unit?", "If a script was previously assigned to this unit,", "it will be replaced.", "Yes", "No", 0, 0) == 1)
							{
								if(!scriptPaths[curUnit].length())
									scriptPaths[curUnit] = "scripts/";
								if(!fileSelect("Please select a script.", scriptPaths[curUnit], "ums;txt", SCREEN_W - 100, SCREEN_H - 100))
									scriptPaths[curUnit] = "";
							}
							else
								scriptPaths[curUnit] = "";
						}
					}
					else
					{
						if(key[global.keyLeft])
							--units[curUnit].getDirection() &= 0xFF;
						if(key[global.keyRight])
							++units[curUnit].getDirection() &= 0xFF;
						if(keyStates[global.keyUp] || keyStates[global.keyDown])
						{
							int oldType = units[curUnit].getType();
							if(key[global.keyUp])
								++oldType;
							else if(oldType)
								--oldType;
							units[curUnit].setType(oldType);
							int graphic;
							if(oldType < playerTypes)
								graphic = 0;
							else if(oldType < enemyTypes + playerTypes)
								graphic = 1;
							else
								graphic = 2;
							units[curUnit].getState() = graphic;
						}
					}
				}
			}
			//Functions keys have in both modes
			if(keyStates[global.keyBoost])
				mode ^= 1;

			//Save and exit
			if(keyStates[KEY_ESC])
			{
				int alertRet = alert3("Save map before before quitting?", 0, 0, "Yes", "No", "Cancel", 0, 0, 0);
				if(alertRet == 2)
				{
					if(alert("Quit editor?", 0, 0, "Yes", "No", 0, 0) == 1)
						break;
				}
				else if(alertRet == 1)
				{
					alertRet = alert3("Save map pre-compiled?", "Pre-compiled should be used for release versions.", "Don't pre-compile if unsure.", "Don't pre-compile", "Pre-compile map", "Cancel", 0, 0, 0);
					if(alertRet == 2)
					{
						if(saveCompiled())
						{
							if(alert("Quit editor?", 0, 0, "Yes", "No", 0, 0) == 1)
								break;
						}
					}
					else if(alertRet == 1)
					{
						if(save())
						{
							if(alert("Quit editor?", 0, 0, "Yes", "No", 0, 0) == 1)
								break;
						}
					}
				}
			}

			map.poll();

			--timerCounter;
		}
		int drawX, drawY;
		if(mode == MODE_MAPEDIT)
		{
			drawX = curTileX * TILESIZE;
			drawY = curTileY * TILESIZE;
			map.draw(drawX, drawY, map.getOffset(curTileX, curTileY), curGraphic);
			rect(global.doublebuffer, curTileX * TILESIZE - drawX, curTileY * TILESIZE - drawY, (curTileX + 1) * TILESIZE - drawX, (curTileY + 1) * TILESIZE - drawY, 0);
		}
		else
		{
			drawX = units[curUnit].getPosX();
			drawY = units[curUnit].getPosY();
			map.draw(drawX, drawY);
		}
		for(i = 0; i < unitsUsed; ++i)
		{
			units[i].draw(drawX, drawY);
			FONT *tFont = global.size12;
			if(i == curUnit)
				tFont = global.size12bold;
			textprintf_ex(global.doublebuffer, tFont, units[i].getPosX() - drawX, units[i].getPosY() - drawY, makecol(0xFF, 0xFF, 0xFF), -1, "%i", units[i].getType());
		}
		if(key[KEY_F1])
		{
			if(mode == MODE_MAPEDIT)
			{
				drawX = curTileX * map.getSmallSize() + (map.getSmallSize() >> 1);
				drawY = curTileY * map.getSmallSize() + (map.getSmallSize() >> 1);
			}
			else
			{
				drawX = units[curUnit].getPosX() / TILESIZE * map.getSmallSize();
				drawY = units[curUnit].getPosY() / TILESIZE * map.getSmallSize();
			}
			blit(map.getMapPreview(), global.doublebuffer, 0, 0, (SCREEN_W - map.getMapPreview()->w) >> 1, (SCREEN_H - map.getMapPreview()->h) >> 1, map.getMapPreview()->w, map.getMapPreview()->h);
			rect(global.doublebuffer, (SCREEN_W - map.getMapPreview()->w) >> 1, (SCREEN_H - map.getMapPreview()->h) >> 1, (SCREEN_W + map.getMapPreview()->w) >> 1, (SCREEN_H + map.getMapPreview()->h) >> 1, 0);
			circlefill(global.doublebuffer, ((SCREEN_W - map.getMapPreview()->w) >> 1) + drawX, ((SCREEN_H - map.getMapPreview()->h) >> 1) + drawY, map.getSmallSize(), makecol(0x00, 0x00, 0xFF));
		}
		if(global.vsync)
			vsync();
		show();
	}
}

string &editorMapClass::getScriptPath(int tileX, int tileY)
{
	return getScriptPath(getOffset(tileX, tileY));
}

string &editorMapClass::getScriptPath(int offset)
{
	return scriptPaths[offset];
}

int editorMapClass::getMapHeight()
{
	return height;
}

int editorMapClass::getMapWidth()
{
	return width;
}

void editorMapClass::setTileset(string filename)
{
	DATAFILE *datafile = load_datafile(filename.c_str());
	if(!datafile)
		errors::datafileError(filename);
	tilesetPath = filename;
	tileset = createDatafileBitmap(datafile, 1);
	if(tileset->w % TILESIZE)
		errors::tilesetLoadError(filename, "Each tile has to be 32 pixel wide");
	tileTypes = tileset->w / TILESIZE;
	unload_datafile(datafile);
}

void editorMapClass::loadMap(ifstream &readfile, string filename)
{
	mapPath = filename;

	preview = NULL;
	//Read header
	char buffer[3];
	readfile.read(buffer, 3);
	if(buffer[0] != MAJOR_VERSION || buffer[1] != MINOR_VERSION || buffer[2] != VERSION_LETTER)
		errors::mapLoadError(filename, "Version mismatch");
	readfile.read(&mapType, 1);

	//Read tileset
	string strBuffer;
	getline(readfile, strBuffer, DELIMITER);
	setTileset(strBuffer);

	//Read size
	readfile.read(i2ch(width), sizeof(width));
	readfile.read(i2ch(height), sizeof(height));
	size = width * height;
	graphicData = new char[size];
	scriptPaths = new string[size];

	//Load map data and scripts
	for(int i = 0; i < size; ++i)
	{
		readfile.read(&graphicData[i], 1);
		//Load script
		readfile.read(&buffer[0], 1);

		//Compile script and pass it to 
		if(buffer[0] == 2)
		{
			getline(readfile, scriptPaths[i], DELIMITER);
		}
		else if(buffer[0] == 1)
		{
//			allegro_message("%i", i);
			errors::editorError("Precompiled scripts are illegal for editing");
		}
	}
	visibleTilesX = SCREEN_W / TILESIZE + 1;
	visibleTilesY = SCREEN_H / TILESIZE + 1;
	maxX = width * TILESIZE - SCREEN_W / 2;
	maxY = height * TILESIZE - SCREEN_H / 2;
}

void editorMapClass::drawTile(int x, int y, char tile)
{
	blit(tileset, global.doublebuffer, tile * TILESIZE, 0, x, y, TILESIZE, TILESIZE);
}


inline int editorMapClass::getOffset(int x, int y)
{
	return x + width * y;
}

void editorMapClass::draw(int &posX, int &posY, int replacePos, int &replaceGraphic)
{
	if(replaceGraphic >= tileTypes)
		replaceGraphic = tileTypes - 1;

	char temp = graphicData[replacePos];
	graphicData[replacePos] = replaceGraphic;
	draw(posX, posY);
	graphicData[replacePos] = temp;
}

void editorMapClass::draw(int &posX, int &posY)
{
	int positionX = MID(SCREEN_W >> 1, posX, maxX);
	int positionY = MID(SCREEN_H >> 1, posY, maxY);
	int minusX = (positionX - (SCREEN_W >> 1)) & (TILESIZE - 1);
	int minusY = (positionY - (SCREEN_H >> 1)) & (TILESIZE - 1);
	int balance = 0;
	if(posX >= (SCREEN_W >>1) + TILESIZE)
		balance = -1;

	int tileX = MID(0, (positionX - minusX) / TILESIZE - visibleTilesX / 2, maxX / TILESIZE) - balance;
	int tileY = MID(0, (positionY - minusY) / TILESIZE - visibleTilesY / 2, maxY / TILESIZE);

	int pos = 0;

	for(int y = 0; y < visibleTilesY + 1; ++y)
	{
		pos = getOffset(tileX, y + tileY);
		for(int x = 0; x < visibleTilesX; ++x)
		{
			if(key[KEY_F2] && !scriptPaths[pos].empty())
				rectfill(global.doublebuffer, x * TILESIZE - minusX, y * TILESIZE - minusY, (x + 1) * TILESIZE - minusX, (y + 1) * TILESIZE - minusY, makecol(0xFF, 0x00, 0x00));
			//else if(pos == replacePos)
			//	drawTile(x * TILESIZE - minusX, y * TILESIZE - minusY, replaceGraphic);
			else
				drawTile(x * TILESIZE - minusX, y * TILESIZE - minusY, graphicData[pos]);
			++pos;
		}
	}
	posX = positionX - (SCREEN_W >> 1);
	posY = positionY - (SCREEN_H >> 1);//tileY * TILESIZE - minusY;
}

void editorMapClass::setTile(int posX, int posY, int newGraphic)
{
	graphicData[getOffset(posX, posY)] = newGraphic;
}

void edUnit::setType(int type)
{
	this->type = type;
}

inline int edUnit::getType()
{
	return type;
}

string editorMapClass::getMapPath()
{
	return mapPath;
}

bool mapEditor::saveCompiled()
{
	return save(true);
}

bool mapEditor::save(bool compiled)
{
	string mapPath = map.getMapPath();
	ofstream mapfile;
	if(fileSelect("Please enter path for saving", mapPath, "msm", SCREEN_W - 100, SCREEN_H - 100))
		mapfile.open(mapPath.c_str(), ios::binary);
	else
		return false;

	mapfile.write((char *)&MAJOR_VERSION, 1);
	mapfile.write((char *)&MINOR_VERSION, 1);
	mapfile.write((char *)&VERSION_LETTER, 1);

	map.save(mapfile, compiled);

	char cBuffer = compiled;
	if(cBuffer)
	{
		//Compute actual path to script file
		string temp = get_filename(mapPath.c_str());
		temp.erase(temp.length() - 4);
		string globScriptPath = string("scripts/") + temp + string("global.ums");

		if(!fileExists(globScriptPath))
			cBuffer = 0;
		mapfile.write(&cBuffer, 1);
		if(cBuffer)
		{
			const int BUFFERSIZE = 0x400;
			unsigned char fullCode[BUFFERSIZE]; //1KB buffer for script data
			int outputSize = BUFFERSIZE;
			if(!compile(globScriptPath, "", fullCode, &outputSize))
				errors::scriptError("An error occured in compile stage.\nSee compiler.log for details.");
			mapfile.write((char *)fullCode, outputSize);
		}
	}
	else
		mapfile.write(&cBuffer, 1);

	cBuffer = campaign;
	mapfile.write(&cBuffer, 1);
	if(campaign)
	{
		mapfile.write(campaignName.c_str(), campaignName.length());
		mapfile.write(&DELIMITER, 1);
	}

	mapfile.write(charsetPath.c_str(), charsetPath.length());
	mapfile.write(&DELIMITER, 1);

	mapfile.write(i2ch(unitsUsed), sizeof(unitsUsed));
	for(int i = 0; i < unitsUsed; ++i)
	{
		char cBuffer = units[i].getType();
		mapfile.write(&cBuffer, 1);

		float fBuffer = units[i].getFlPosX();
		mapfile.write(f2ch(fBuffer), sizeof(fBuffer));
		fBuffer = units[i].getFlPosY();
		mapfile.write(f2ch(fBuffer), sizeof(fBuffer));

		cBuffer = units[i].getDirection();
		mapfile.write(&cBuffer, 1);
		//Set initstate to 0, because its used for animation atm anyway
		cBuffer = 0;
		mapfile.write(&cBuffer, 1);
		if(scriptPaths[i].length())
		{
			if(!compiled)
			{
				cBuffer = 2;
				mapfile.write(&cBuffer, 1);
				scriptPaths[i] = string("scripts/") + string(get_filename(scriptPaths[i].c_str()));
				mapfile.write(scriptPaths[i].c_str(), scriptPaths[i].length());
				mapfile.write(&DELIMITER, 1);
			}
			else
			{
				cBuffer = 1;
				mapfile.write(&cBuffer, 1);
				const int BUFFERSIZE = 0x400;
				unsigned char fullCode[BUFFERSIZE]; //1KB buffer for script data
				int outputSize = BUFFERSIZE;
				if(!compile(scriptPaths[i], "", fullCode, &outputSize))
					errors::scriptError("An error occured in compile stage.\nSee compiler.log for details.");
				mapfile.write((char *)fullCode, outputSize);
			}
		}
		else
		{
			cBuffer = 0;
			mapfile.write(&cBuffer, 1);
		}
	}

	return true;
}

void editorMapClass::save(ofstream &mapfile, bool compiled)
{
	mapfile.write(&mapType, 1);
	char cBuffer;
	mapfile.write(tilesetPath.c_str(), tilesetPath.length());
	mapfile.write(&DELIMITER, 1);
	mapfile.write(i2ch(width), sizeof(width));
	mapfile.write(i2ch(height), sizeof(height));
	for(int i = 0; i < size; ++i)
	{
		mapfile.write(&graphicData[i], 1);
		if(scriptPaths[i].length())
		{
			if(!compiled)
			{
				cBuffer = 2;
				mapfile.write(&cBuffer, 1);
				scriptPaths[i] = string("scripts/") + string(get_filename(scriptPaths[i].c_str()));
				mapfile.write(scriptPaths[i].c_str(), scriptPaths[i].length());
				mapfile.write(&DELIMITER, 1);
			}
			else
			{
				cBuffer = 1;
				const int BUFFERSIZE = 0x400;
				unsigned char fullCode[BUFFERSIZE]; //1KB buffer for script data
				int outputSize = BUFFERSIZE;
				mapfile.write(&cBuffer, 1);
				if(!compile(scriptPaths[i], "", fullCode, &outputSize))
					errors::scriptError("An error occured in compile stage.\nSee compiler.log for details.");
				mapfile.write((char *)fullCode, outputSize);
			}
		}
		else
		{
			cBuffer = 0;
			mapfile.write(&cBuffer, 1);
		}
	}
}

BITMAP *editorMapClass::getMapPreview(int smallSizeX, int smallSizeY)
{
	if(preview && refreshCountdown > 0)
		return preview;
	BITMAP *temp = tileset;

	smallSize = smallSizeX / width;
	if(smallSizeY / height < smallSize)
		smallSize = smallSizeY / height;
	tileset = createBitmap(smallSize * tileTypes, smallSize);
	stretch_blit(temp, tileset, 0, 0, temp->w, temp->h, 0, 0, tileset->w, tileset->h);

	preview = createBitmap(smallSize * width, smallSize * height);

	//int playerTileX = playerPosX >> 5, playerTileY = playerPosY >> 5;

	for(int i = 0; i < width; ++i)
	{
		for(int j = 0; j < height; ++j)
		{
			drawTile(i * smallSize, j * smallSize, graphicData[getOffset(i, j)], smallSize, preview);
		}
	}

	destroy_bitmap(tileset);
	tileset = temp;
	refreshCountdown = 128;
	return preview;
}

inline void editorMapClass::drawTile(int x, int y, char tile, int tilesize, BITMAP *dest)
{
	blit(tileset, dest, tile * tilesize, 0, x, y, tilesize, tilesize);
}

int editorMapClass::getSmallSize()
{
	return smallSize;
}

void editorMapClass::poll()
{
	if(refreshCountdown)
		--refreshCountdown;
}
